package jp.hishidama.ant.types.htlex;

import java.lang.reflect.Method;
import java.util.*;

import jp.hishidama.html.lexer.token.AttributeToken;
import jp.hishidama.html.lexer.token.Tag;
import jp.hishidama.html.parser.elem.HtElement;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.taskdefs.condition.*;
import org.apache.tools.ant.types.DataType;

/**
 * HtHtmlLexer^O^Cv.
 *<p>
 * HTMLt@C̃^Ȍfsf[^^CvB
 * </p>
 *
 * @author <a target="hishidama" href=
 *         "http://www.ne.jp/asahi/hishidama/home/tech/soft/java/ant/htlex.html"
 *         >Ђ</a>
 * @since 2009.01.20
 * @version 2009.02.17
 */
public class TagType extends DataType {

	protected MatchEnum nameMatch;

	/**
	 * Oݒ.
	 *
	 * @param name
	 *            ^O
	 */
	public void setName(String name) {
		if (nameMatch == null) {
			nameMatch = new MatchEnum(MatchEnum.IGNORE_CASE);
		}
		nameMatch.setPattern(name);
	}

	/**
	 * O}b`O@ݒ.
	 *
	 * @param match
	 *            }b`O@
	 */
	public void setNameMatch(MatchEnum match) {
		if (nameMatch != null) {
			match.setPattern(nameMatch.getPatternString());
		}
		nameMatch = match;
	}

	protected String tago, tagc;

	/**
	 * ^OJݒ.
	 *
	 * @param s
	 *            ^OJ
	 */
	public void setTagO(String s) {
		tago = s;
	}

	/**
	 * ^Oݒ.
	 *
	 * @param s
	 *            ^O
	 */
	public void setTagC(String s) {
		tagc = s;
	}

	protected PairTagEnum pair = new PairTagEnum(PairTagEnum.IGNORE);

	/**
	 * yAݒ.
	 *
	 * @param p
	 *            yA
	 */
	public void setPair(PairTagEnum p) {
		this.pair = p;
	}

	protected Condition attrCond = null;

	/**
	 * ǉ.
	 *
	 * @param cond
	 *            
	 */
	public void addConfigured(Condition cond) {
		if (attrCond == null) {
			attrCond = cond;
		} else if (attrCond instanceof And) {
			((And) attrCond).add(cond);
		} else {
			And and = new And();
			and.setProject(this.getProject());
			and.add(attrCond);
			and.add(cond);
			attrCond = and;
		}
	}

	/**
	 * Andǉ.
	 *
	 * @param cond
	 *            
	 */
	public void addConfiguredAnd(And cond) {
		addConfigured(cond);
	}

	/**
	 * Orǉ.
	 *
	 * @param cond
	 *            
	 */
	public void addConfiguredOr(Or cond) {
		addConfigured(cond);
	}

	/**
	 * Notǉ.
	 *
	 * @param cond
	 *            
	 */
	public void addConfiguredNot(Not cond) {
		addConfigured(cond);
	}

	protected List<TagNewType> tagList = new ArrayList<TagNewType>();

	/**
	 * ^Oϊǉ.
	 *
	 * @param tag
	 *            ^Oϊ^Cv
	 */
	public void addConfigured(TagNewType tag) {
		tagList.add(tag);
	}

	protected HtLexerConverter converter;

	/**
	 * s.
	 *
	 * @param conv
	 *            Ro[^[
	 * @throws BuildException
	 *             G[
	 */
	public void validate(HtLexerConverter conv) throws BuildException {
		converter = conv;

		if (pair.getIndex() >= PairTagEnum.PARSER_INDEX) {
			converter.setUseParser(true);
		}

		if (attrCond != null) {
			validateCondition(attrCond);
		}

		for (TagNewType tt : tagList) {
			tt.validate(conv);
		}
	}

	protected void validateCondition(Condition c) throws BuildException {
		if (c == null) {
			throw new BuildException("condition must be set.", getLocation());
		}
		if (c instanceof AttrCondType) {
			AttrCondType ac = (AttrCondType) c;
			ac.validate(this);
			return;
		}

		// and,or,not̂݋
		if (c instanceof And || c instanceof Or || c instanceof Not) {
			Enumeration<Condition> n = getBaseConditions(c);
			while (n.hasMoreElements()) {
				validateCondition(n.nextElement());
			}
			return;
		} else {
			throw new BuildException(c.getClass().getSimpleName()
					+ " can not be used.", getLocation());
		}
	}

	protected Method GetConditionMethod;

	@SuppressWarnings("unchecked")
	protected Enumeration<Condition> getBaseConditions(Object obj) {
		try {
			if (GetConditionMethod == null) {
				GetConditionMethod = ConditionBase.class.getDeclaredMethod(
						"getConditions", (Class[]) null);
				GetConditionMethod.setAccessible(true);
			}
			return (Enumeration<Condition>) GetConditionMethod.invoke(obj,
					(Object[]) null);
		} catch (Exception e) {
			throw new BuildException(e, getLocation());
		}
	}

	/**
	 * ^Oϊ.
	 *
	 * @param tag
	 *            ^O
	 * @param he
	 *            ^ȎĂvfi͂ĂȂꍇAnullj
	 * @return eύXꍇAtrue
	 */
	public boolean convert(Tag tag, HtElement he) {
		boolean ret = false;
		if (matches(tag, he)) {
			converter.logMatch(tag);
			for (TagNewType tt : tagList) {
				ret |= tt.convert(tag, he);
			}
		}
		return ret;
	}

	protected List<AttributeToken> tokenList;

	/**
	 * Ώۑꗗ擾.
	 *
	 * @return ꗗ
	 * @see AttrCondType#eval()
	 */
	public List<AttributeToken> getTokenList() {
		return tokenList;
	}

	/**
	 * }b`Of.
	 *
	 * @param tag
	 *            ^O
	 * @param he
	 *            ^ȎĂvfi͂ĂȂꍇAnullj
	 * @return w肳ĂɃ}b`ꍇAtrue
	 */
	public boolean matches(Tag tag, HtElement he) {
		String name = tag.getName();
		if (!equalsName(name)) {
			return false;
		}
		if (!equalsTagO(tag.getTag1())) {
			return false;
		}
		if (!equalsTagC(tag.getTag2())) {
			return false;
		}
		if (attrCond != null) {
			tokenList = tag.getAttributeList();
			if (!attrCond.eval()) {
				return false;
			}
		}
		if (!pair.matches(tag, he)) {
			return false;
		}
		return true;
	}

	protected boolean equalsName(String s) {
		if (nameMatch == null) {
			return true;
		}
		return nameMatch.matches(s);
	}

	protected boolean equalsTagO(String t) {
		if (tago == null) {
			return true;
		}
		return tago.equals(t);
	}

	protected boolean equalsTagC(String t) {
		if (tagc == null) {
			return true;
		}
		return tagc.equals(t);
	}
}
