package jp.hishidama.html.lexer.rule;

import java.io.IOException;
import java.util.*;

import jp.hishidama.html.lexer.reader.Char;
import jp.hishidama.html.lexer.reader.CharType;
import jp.hishidama.html.lexer.token.*;

/**
 * HtHtmlLexer[.
 *<p>
 * HTML̃^OEEeLXg̉߂sNXB
 * </p>
 *
 * @author <a target="hishidama" href=
 *         "http://www.ne.jp/asahi/hishidama/home/tech/soft/java/htlexer.html"
 *         >Ђ</a>
 * @since 2009.01.10
 * @version 2009.02.06
 */
public abstract class HtLexerRule {

	protected HtLexer data;

	protected HtLexerRule(HtLexer data) {
		this.data = data;
	}

	protected class Chars {
		private List<Char> d = new ArrayList<Char>();

		/**
		 * RXgN^[.
		 */
		public Chars() {
		}

		/**
		 * RXgN^[.
		 *
		 * @param c
		 *            
		 */
		public Chars(Char c) {
			d.add(c);
		}

		/**
		 * 擾.
		 * <p>
		 * CfbNXɎw肳ꂽԍ܂ŕǂݍށB
		 * </p>
		 *
		 * @param n
		 *            CfbNX
		 * @return iKnullȊOj
		 * @throws IOException
		 */
		public Char get(int n) throws IOException {
			for (int i = d.size(); i <= n; i++) {
				d.add(data.readChar());
			}
			return d.get(n);
		}

		public boolean isType(CharType[] types) throws IOException {
			for (int i = 0; i < types.length; i++) {
				if (get(i).getType() != types[i]) {
					return false;
				}
			}
			return true;
		}

		public StringBuilder clear(int use) {
			StringBuilder sb = new StringBuilder();
			for (int i = 0; i < use; i++) {
				sb.append(d.get(i).getSB());
			}
			unread(use);
			return sb;
		}

		protected void unread(int n) {
			for (int i = d.size() - 1; i >= n; i--) {
				data.unreadChar(d.get(i));
			}
			d.clear();
		}
	}

	/**
	 * [ɏ]߂sB
	 *
	 * @return g[N
	 * @throws IOException
	 */
	public abstract Token parse() throws IOException;

	protected SeparateRule sepRule;

	/**
	 * ؂胋[ݒ.
	 *
	 * @param sepRule
	 *            ؂胋[
	 */
	protected void setSepRule(SeparateRule sepRule) {
		this.sepRule = sepRule;
	}

	/**
	 * ؂蕶f.
	 *
	 * @param ch
	 *            
	 * @return ߂I镶̏ꍇAtrue
	 * @throws IOException
	 */
	protected boolean parseEnd(Char ch) throws IOException {
		// if (sepRule != null) {
		return sepRule.isEnd(ch);
		// }
		// return false;
	}

	/**
	 * XLbv.
	 * <p>
	 * Xy[XE^uEŝ݂𒊏oB
	 * </p>
	 *
	 * @return XLbvg[Niꍇnullj
	 * @throws IOException
	 */
	public SkipToken skipSpace() throws IOException {
		SkipToken skip = null;
		loop: for (;;) {
			Char ch = data.readChar();
			switch (ch.getType()) {
			case SPACE:
				if (skip == null) {
					skip = createSkipToken();
				}
				skip.add(new SpaceAtom(ch.getSB()));
				continue;
			case EOL:
				if (skip == null) {
					skip = createSkipToken();
				}
				skip.add(new CrLfAtom(ch.getSB()));
				continue;
			default:
				data.unreadChar(ch);
				break loop;
			}
		}
		return skip;
	}

	protected SkipToken createSkipToken() {
		return new SkipToken();
	}
}
